@@ -23,6 +23,7 @@ import (
 	"context"
 	"fmt"
 	"reflect"
+	"sort"
 	"strings"
 	"testing"
 
@@ -165,12 +166,39 @@ func TestEnsureInternalInstanceGroupsLimit(t *testing.T) {
 	igName := makeInstanceGroupName(vals.ClusterID)
 	_, err = gce.ensureInternalInstanceGroups(igName, nodes)
 	require.NoError(t, err)
-
 	instances, err := gce.ListInstancesInInstanceGroup(igName, vals.ZoneName, allInstances)
 	require.NoError(t, err)
 	assert.Equal(t, maxInstancesPerInstanceGroup, len(instances))
 }
 
+func TestEnsureMultipleInstanceGroups(t *testing.T) {
+	t.Parallel()
+
+	vals := DefaultTestClusterValues()
+	gce, err := fakeGCECloud(vals)
+	require.NoError(t, err)
+	gce.AlphaFeatureGate = NewAlphaFeatureGate([]string{AlphaFeatureSkipIGsManagement})
+
+	nodes, err := createAndInsertNodes(gce, []string{"n1"}, vals.ZoneName)
+	require.NoError(t, err)
+
+	baseName := makeInstanceGroupName(vals.ClusterID)
+	clusterIGs := []string{baseName, baseName + "-1", baseName + "-2", baseName + "-3"}
+	for _, igName := range append(clusterIGs, "zz-another-ig", "k8s-ig--cluster2-id") {
+		ig := &compute.InstanceGroup{Name: igName}
+		err := gce.CreateInstanceGroup(ig, vals.ZoneName)
+		require.NoError(t, err)
+	}
+
+	igsFromCloud, err := gce.ensureInternalInstanceGroups(baseName, nodes)
+	require.NoError(t, err)
+	assert.Len(t, igsFromCloud, len(clusterIGs), "Incorrect number of Instance Groups")
+	sort.Strings(igsFromCloud)
+	for i, igName := range clusterIGs {
+		assert.True(t, strings.HasSuffix(igsFromCloud[i], igName))
+	}
+}
+
 func TestEnsureInternalLoadBalancer(t *testing.T) {
 	t.Parallel()
 
@@ -525,6 +553,29 @@ func TestEnsureInternalLoadBalancerDeleted(t *testing.T) {
 	assertInternalLbResourcesDeleted(t, gce, svc, vals, true)
 }
 
+func TestSkipInstanceGroupDeletion(t *testing.T) {
+	t.Parallel()
+
+	vals := DefaultTestClusterValues()
+	gce, err := fakeGCECloud(vals)
+	require.NoError(t, err)
+
+	svc := fakeLoadbalancerService(string(LBTypeInternal))
+	svc, err = gce.client.CoreV1().Services(svc.Namespace).Create(context.TODO(), svc, metav1.CreateOptions{})
+	require.NoError(t, err)
+	_, err = createInternalLoadBalancer(gce, svc, nil, []string{"test-node-1"}, vals.ClusterName, vals.ClusterID, vals.ZoneName)
+	assert.NoError(t, err)
+
+	gce.AlphaFeatureGate = NewAlphaFeatureGate([]string{AlphaFeatureSkipIGsManagement})
+	err = gce.ensureInternalLoadBalancerDeleted(vals.ClusterName, vals.ClusterID, svc)
+	assert.NoError(t, err)
+
+	igName := makeInstanceGroupName(vals.ClusterID)
+	ig, err := gce.GetInstanceGroup(igName, vals.ZoneName)
+	assert.NoError(t, err)
+	assert.NotNil(t, ig, "Instance group should not be deleted when flag 'NetLB_RBS' is present")
+}
+
 func TestEnsureInternalLoadBalancerDeletedTwiceDoesNotError(t *testing.T) {
 	t.Parallel()
 
